from flask import Blueprint,request,current_app

menu = Blueprint('menu',__name__,
                template_folder='',
                url_prefix="menu")
                                    


from . import views
from . import models


@menu.before_app_request
def add_menu_context():
    if not 'static' in request.path and not 'templates' in request.path:
        links = models.MenuLink.query.all()
        for link in links:
            link.active = False
            if request.endpoint == link.endpoint:
                link.active = True
            link.save()
        current_app.jinja_env.globals['nav_links'] = links




from wtforms import fields, widgets

class CKTextEditorWidget(widgets.TextArea):
    def __call__(self,field,**kwargs):
        if kwargs.get('class_',False):
            kwargs['class_'] += ' ckeditor'
        else:
            kwargs['class_'] = 'ckeditor'
        kwargs['rows'] = '8'
        return super(CKTextEditorWidget,self).__call__(field,**kwargs)

class CKTextEditorField(fields.TextAreaField):
    widget = CKTextEditorWidget()



from flask import Blueprint
import os
from os import path as op

exists = lambda x: op.exists(op.join(os.getcwd(),x))

page = Blueprint('page',__name__,
                template_folder='templates',
                static_url_path='/_page_static',
                static_folder=os.path.abspath(os.path.dirname(__file__)),
                url_prefix='/page')
'''
@page.before_app_request
def add_pages():
    pages = Page._session.query(Page).filter(Page.add_to_nav==True).all()
    if flask.request.view_args:
        slug = flask.request.view_args.get('slug')
    else:
        slug = None
    if slug is None:
        slug = flask.request.endpoint
    for page in pages:
        if page.slug == slug:
            page._current = True
        else:
            page._current = False
    app.jinja_env.globals['page_width'] = '-fluid'
    app.jinja_env.globals['pages'] = pages
    app.jinja_env.globals['slug'] = slug
'''
@page.before_app_request
def get_contact_us_data():
    #data = json.loads(open(os.path.join(base,'contact_data.json'),'r').read())
    #app.jinja_env.globals['data'] = data
    pass



from admin import admin
from importlib import import_module
from flask import current_app
from .models import Setting,Type
from flask.ext.wtf import Form


class BaseSettings(object):
    _settings = []
    _values = {}
    _form = None
    _defaults = {}
    _cfg = ''

    def __init__(self):
        self._cfg = current_app.config
        for itm in self._settings:
            if not itm in self._values:
                try:
                    self.set_setting(itm,self._defaults[itm])
                except KeyError:
                    raise ValueError('Need to be given a value or default for {}'.format(itm))
            self.set_setting(itm,self._cfg[itm])

    def _get_setting_field(self,setting,field_type=None):
        pass

    def _get_form(self):
        raise NotImplementedError

    @property
    def form(self):
        return self._get_form()

    def set_setting(self,setting,value):
        if setting in self._settings:
            self._values[setting] = value

    def get_setting(self,setting,default=None):
        if setting in self._settings:
            return self._values[setting]
        return default

    def apply_settings(self,**kwargs):
        if len(kwargs) > 0:
            for k in kwargs:
                self.set_setting(k,kwargs[k])
        for setting in self._settings:
            if not setting.startswith('_'):
                self._cfg[setting.upper()] = self.get_setting(setting)

    def _get_default_field(self,setting,fields):
        return fields.__dict__['StringField']

class SiteSettings(BaseSettings):
    _settings = [
            'RECAPTCHA_PUBLIC_KEY',
            'RECAPTCHA_PRIVATE_KEY',
            'ADMIN_PER_PAGE',
            'CODEMIRROR_LANGUAGES',
            'CODEMIRROR_THEME',
            'BLUEPRINTS',
            'EXTENSIONS',
            'TEMPLATE_FILTERS',
            'CONTEXT_PROCESSORS',
    ]
    _defaults = {
            'RECAPTCHA_PUBLIC_KEY':'',
            'RECAPTCHA_PRIVATE_KEY':'',
            'ADMIN_PER_PAGE':'',
            'CODEMIRROR_LANGUAGES':['python'],
            'CODEMIRROR_THEME':'3024-year',
            'BLUEPRINTS':'',
            'EXTENSIONS':'',
            'TEMPLATE_FILTERS':'',
            'CONTEXT_PROCESSORS':'',
    }
    def _get_setting_field(self,setting,field_type=None):
        fields = import_module('wtforms.fields')
        if field_type is not None:
            field = fields.__dict__[field_type]
        else:
            from .models import Setting
            s = Setting.query.filter(Setting.name==setting).first()
            if s is None:
                field = self._get_default_field(setting,fields)
            else:
                field = fields.__dict__[s.type.field_type]
        return field

    def _get_form(self):
        form_args = {}
        for itm in self._settings:
            form_args[itm] = self._get_setting_field(itm)()
        self._form = type(
            'EditSiteSettingsForm',(Form,),form_args
        )
        return self._form






@admin.before_app_request
def add_settings():
    from app import app
    settings = app.config.copy()
    CACHED_SETTINGS = [
            'RECAPTCHA_PUBLIC_KEY',
            'RECAPTCHA_PRIVATE_KEY',
            'ADMIN_PER_PAGE',
            'CODEMIRROR_LANGUAGES',
            'CODEMIRROR_THEME',
            'BLUEPRINTS',
            'EXTENSIONS',
            'TEMPLATE_FILTERS',
            'CONTEXT_PROCESSORS',
    ]
    for itm in CACHED_SETTINGS:
        setting = Setting.query.filter(Setting.name==itm).first()
        if setting is None:
            t = Type.query.filter(Type.name==type(settings[itm])).first()
            value = settings.get(itm,None)
            if value is None:
                value = ''
            if t is None:
                t = Type(type(settings[itm]))
                t.save()
            setting = Setting(
                    name=itm,type=t,value=value
            )
            setting.save()



from ...basemodels import BaseMixin
import sqlalchemy as sa

class Menu(BaseMixin):

    name = sa.Column(sa.String(255),unique=True)
    

class MenuLink(BaseMixin):

    name = sa.Column(sa.String(255),nullable=False)
    menu = sa.orm.relationship('Menu',backref=sa.orm.backref('links',lazy='dynamic'))
    menu_id = sa.Column(sa.Integer,sa.ForeignKey('menus.id'))
    text = sa.Column(sa.String(255))
    endpoint = sa.Column(sa.String(255))








from flask_xxl.basemodels import BaseMixin
from flask import url_for
from LoginUtils import encrypt_password, check_password
from datetime import datetime
from sqlalchemy import Column,String,Integer,Boolean,Date,DateTime,ForeignKey,UnicodeText,Table
from sqlalchemy.orm import relationship,backref


#import sqlalchemy to global namespace

class UnknownUser(object):
    is_unknown = True

class Role(BaseMixin):

    name = Column(String(255))
    can_view = Column(Boolean,default=True,nullable=False)
    can_add = Column(Boolean,default=False,nullable=False)
    can_edit = Column(Boolean,default=False,nullable=False)
    can_delete = Column(Boolean,default=False,nullable=False)

class User(BaseMixin):

    username = Column(String(255),unique=True,nullable=False)
    first_name = Column(String(255),default="")
    last_name = Column(String(255),default="")
    email = Column(String(255),nullable=False,unique=True)
    role_id = Column(Integer,ForeignKey('roles.id'))
    role = relationship('Role',backref=backref(
                    'users',lazy='dynamic'))
    add_date = Column(DateTime,default=datetime.now)
    _pw_hash = Column(UnicodeText,nullable=False)
    age = Column(Integer)


    def __init__(self,*args,**kwargs):
        if 'first_name' in kwargs:
            self.first_name = kwargs.pop('first_name')
        if 'last_name' in kwargs:
            self.last_name = kwargs.pop('last_name')
        if 'email' in kwargs:
            self.email = kwargs.pop('email')
        if 'role' in kwargs:
            self.role = kwargs.pop('role')
        if 'role_id' in kwargs:
            self.role_id = kwargs.pop('role_id')
        if 'password' in kwargs:
            self.password = kwargs.pop('password')

    @property
    def is_unknown(self):
        return False
    
    def check_password(self, pw):
        return check_password(pw,self._pw_hash)

    @classmethod
    def get_by_email(cls, email):
        return cls.query.filter_by(email=email).first()

    @property
    def password(self):
        raise ValueError('Private Value!!!!')

    @password.setter
    def password(self,pw):
        self._pw_hash = encrypt_password(pw)

    @property
    def full_name(self):
        return '{} {}'.format(self.first_name.title(),self.last_name.title())

    def __str__(self):
        if self.first_name != "":
            rtn = self.full_name
        else:
            rtn = self.email
        return rtn

    def __repr__(self):
        return 'User<{} {}'.format(self.email,self.first_name)

    def _get_absolute_url(self):
        return url_for('member.profile',member_id=str(int(self.id)))

    def _get_edit_url(self):
        return '#'




# -*- coding: utf-8 -*-

"""
    admin.views

    Administrative Views
"""
import os
from ..auth.utils import login_required
from datetime import datetime
from flask_wtf import Form
from wtforms.ext.sqlalchemy.orm import model_form
from ...baseviews import BaseView,ModelView
from ..admin import admin
from flask import request,session,flash
from .forms import AddBlogForm

class FakePage(object):
    link = None
    text = None

    def __init__(self,page):
        self.text = page.title
        self.link = page._get_page_url()

class AdminDashboardView(BaseView):
    _template = 'dashboard.html'

    decorators = [login_required]

    def get(self):

        return self.render()

class AdminSiteSettingsView(BaseView):
    _template = 'settings.html'

    decorators = [login_required]

    def get(self):
        from admin.forms import SiteSettingsForm
        from admin.models import SiteSetting
        settings = SiteSettings.query.all()
        self._form = SiteSettingsForm(obj=settings)



class AdminPageView(BaseView):
    _template = 'add.html'
    _context = {
                    'form_args':
            {
                'heading':'Add CMS Page',
            }
    }

    decorators = [login_required]

    def get(self):
        if not 'content' in request.endpoint:
            from ..page.models import Page
            from wtforms import FormField
            from .forms import BaseTemplateForm
            from ..auth.models import User
            from ..page.models import Template
            self._context['choices'] = [(x,x.name) for x in Template.query.all()]
            from ext import db
            form = model_form(Page,db.session,base_class=Form,exclude=['date_added','added_by'])
            class PageForm(form):
                template = FormField(BaseTemplateForm)
                def __init__(self,*args,**kwargs):
                    if 'prefix' in kwargs:
                        kwargs.pop('prefix')
                    super(PageForm,self).__init__(*args,**kwargs)

            self._form = PageForm(prefix=False)
        else:
            from ..page.forms import EditContentForm
            from ..auth.models import User
            self._form = EditContentForm
        return self.render()

    def post(self):
        if 'content' in request.endpoint:
            session['content'] = request.form['content'][:]
            return self.redirect('admin.add_page')
        else:
            from ..auth.models import User
            from .forms import BaseTemplateForm
            from ..page.models import Template
            from ..page.models import Page
            from wtforms import FormField
            from ...ext import db
            self._context['choices'] = [(x,x.name) for x in Template.query.all()]
            form = model_form(Page,db.session,base_class=Form,exclude=['date_added','added_by'])
            class PageForm(form):
                template = FormField(BaseTemplateForm)

                def _get_template_id(self,template_name):
                    from page.models import Template
                    t = Template.query.filter(Template.name==template_name).first()
                    if t:
                        return t.id
                    return None

                def __init__(self,*args,**kwargs):
                    if 'prefix' in kwargs:
                        kwargs.pop('prefix')
                    super(PageForm,self).__init__(*args,**kwargs)
                def populate_obj(self,obj):
                    for name,field in self._fields.items():
                        if name == 'template':
                            obj.template_id = self._get_template_id(field.template.data)
                            self._fields.pop(name)
                    super(PageForm,self).populate_obj(obj)

            self._form = PageForm(request.form)
            page = Page()
            self._form.populate_obj(page)
            if 'content' in session:
                page.content = session.pop('content',None)
            page.save()
            flash('Successfully created cms page: {}'.format(page.name))
            return self.redirect('admin.pages')
        return self.render()




class AdminTemplateView(BaseView):
    _template = 'add.html'
    _context = {'mce':True,
                    'form_args':
            {
                'heading':'Add a Template',
            }
    }

    decorators = [login_required]

    def get(self):
        from auth.models import User
        from page.models import Template
        from wtforms import FormField
        from admin.forms import TemplateBodyFieldForm
        from ext import db
        AddTemplateForm = model_form(Template,db.session,base_class=Form,exclude=['blocks','pages','body'])
        class TemplateForm(AddTemplateForm):
            body = FormField(TemplateBodyFieldForm)

        self._form = TemplateForm
        return self.render()

    def post(self):
        from page.models import Template
        from auth.models import User
        from ext import db
        self._form = model_form(Template,db.session,base_class=Form,exclude=['blocks','pages','body'])(request.form)
        if self._form.validate():
            template = Template()
            self._form.populate_obj(template)
            template.save()
            filename = template.filename
            if template.body is not None:
                body = template.body[:]
            else:
                body = ''
            from settings import BaseConfig
            import os
            templatedir = os.path.join(BaseConfig.ROOT_PATH,'templates')
            os.system('touch {}'.format(os.path.join(templatedir,filename)))
            fp = open(os.path.join(templatedir,filename),'w')
            fp.write(body+'\n')
            fp.close()
            flash('you have created a new template named: {}'.format(template.name))
            return self.redirect('admin.templates')
        else:
            flash('You need to give the template a name')
        return self.render()

    @staticmethod
    @admin.before_app_request
    def check_templates():
        from ..page.models import Template
        from ..auth.models import User
        from flask import current_app
        templates = Template.query.all()
        template_dir = current_app.config['ROOT_PATH'] + '/' + 'templates'
        if not os.path.exists(template_dir):
            os.mkdir(template_dir)
        names = [t.name for t in templates]
        for t in os.listdir(template_dir):
            if not t in names:
                temp = Template()
                temp.name = t
                temp.filename = os.path.join(template_dir,t)
                temp.body = open(os.path.join(template_dir,t),'r').read()
                temp.save()

class AdminBlockView(BaseView):
    _template = 'add.html'
    _context = {'mce':True,
                    'form_args':
            {
                'heading':'Add CMS Block',
            }
    }

    decorators = [login_required]

    def get(self):
        if not 'content' in request.endpoint:
            from auth.models import User
            from page.models import Block
            from ext import db
            AddBlockForm = model_form(Block,db.session,base_class=Form)
            self._form = AddBlockForm
        else:
            content = session.pop('content','')
            from page.forms import EditContentForm
            self._form = EditContentForm
            self._form.content.data = content
        return self.render()

    def post(self):
        if 'content' in request.endpoint:
            session['content'] = request.form['content'][:]
            return self.redirect('admin.add_block')
        else:
            content = session.pop('content',None)
            from page.models import Block
            from ext import db
            AddBlockForm = model_form(Block,db.session,base_class=Form)
            self._form = AddBlockForm(request.form)
            block = Block()
            self._form.populate_obj(block)
            if 'content' in session:
                block.content = content
            block.save()
            flash('Successfully created cms block: {}'.format(block.name))
            return self.redirect('admin.blocks')
        return self.render()
    @staticmethod
    @admin.before_app_request
    def check_blocks():
        from ..page.models import Block as model
        from ..auth.models import User
        objs = model.query.all()
        from flask import current_app as app
        block_dir = app.config['ROOT_PATH'] + '/' + 'blocks'
        if not os.path.exists(block_dir):
            os.mkdir(block_dir)
        names = [o.name for o in objs]
        for o in os.listdir(block_dir):
            if not o in names:
                if not o.startswith('_'):
                    temp = model()
                    temp.name = o
                    temp.content = ''.join(map(str,open(os.path.join(block_dir,o),'r').readlines()))
                    temp.save()

class AdminCMSListView(BaseView):
    _template = 'list.html'
    _context = {}

    decorators = [login_required]

    def get(self):
        from settings import BaseConfig
        from page.models import Page,Block,Template
        from auth.models import User
        pp = BaseConfig.ADMIN_PER_PAGE
        if 'users' in request.endpoint:
            obj= User
        if 'page' in request.endpoint:
            obj = Page
        if 'block' in request.endpoint:
            obj = Block
        if 'template' in request.endpoint:
            obj = Template
        objs = obj.query.all()
        if len(objs) > pp:
            return self.redirect('admin.page_{}'.format(obj.__name__.lower()),page_num=1)
        exclude = ['password','templates','pages','id','body','content',
                    'query','metadata','template','blocks','body_body',
                    'template_id','use_base_template','absolute_url','body-body',
                    'added_by','body_body','articles','role_id','_pw_hash',
                    'is_unknown']
        headings = obj.get_all_columns(exclude)
        self._context['objs'] = objs
        self._context['columns'] = [x[0] for x in headings]
        self._context['headings'] = [x[1] for x in headings]
        return self.render()


class AdminListPageView(BaseView):
    _template = 'list.html'

    decorators = [login_required]

    def get(self,page_num=1):
        from admin.utils import Pagination
        from settings import BaseConfig
        pp = BaseConfig.ADMIN_PER_PAGE
        endpoint = request.endpoint
        if '_users' in endpoint:
            from auth.models import User
        if '_page' in endpoint:
            from page.models import Page as model
        elif '_template' in endpoint:
            from page.models import Template as model
        else:
            from page.models import Block as model
        p = Pagination(model,page_num)
        self._context['objs'] = p._objs
        self._context['pagination'] = p
        exclude = ['password','templates','pages','id','body','content',
                    'query','metadata','template','blocks','body-body',
                    'template_id','use_base_template','absolute_url','body_body',
                    'added_by','articles','role_id','_pw_hash','is_unknown']
        headings = p._objs[0].get_all_columns(exclude)
        self._context['columns'] = [x[0] for x in headings]
        self._context['headings'] = [x[1] for x in headings]
        self._context['use_codemirror'] = True
        return self.render()





class AdminDetailView(BaseView):

    decorators = [login_required]

    def get(self,slug=None,name=None):
        endpoint = request.endpoint[:]
        if 'page' in request.endpoint:
            self._template = 'view_page.html'
        if 'block' in request.endpoint:
            self._template = 'view_block.html'
        if 'template' in request.endpoint:
            self._template = 'view_template.html'
        if 'page' in endpoint:
            from page.models import Page
            obj = Page.query.filter(Page.slug==slug).first()
        if 'block' in endpoint:
            from page.models import Block
            obj = Block.query.filter(Block.name==name).first()
        if 'template' in endpoint:
            from page.models import Template
            obj = Template.query.filter(Template.name==name).first()

        self._context['obj'] = obj
        self._context['lines'] = obj.content.split('\n') or obj.body.split('\n')
        return self.render()



class AdminEditView(BaseView):
    _template = 'add.html'
    _context = {
                    'form_args':
            {
                'heading':'Edit CMS Item',
            }
    }

    decorators = [login_required]

    def get(self,item_id=None):
        from auth.models import User
        from ext import db
        if 'page' in request.endpoint:
            from page.models import Page
            page = Page.get_by_id(item_id)
            if not 'content' in request.endpoint:
                self._form = model_form(Page,db.session,base_class=Form,exclude=['added_by','date_added'])
                self._context['obj'] = page
            else:
                from page.forms import EditContentForm
                self._form = EditContentForm
            self._form_obj = page
        elif 'block' in request.endpoint:
            from page.models import Block
            block = Block.get_by_id(item_id)
            if not 'content' in request.endpoint:
                self._form = model_form(Block,db.session,base_class=Form,exclude=['templates','pages'])
                self._context['obj'] = block
            else:
                from page.forms import EditContentForm
                self._form = EditContentForm
            self._form_obj = block
        else:
            from admin.forms import TemplateBodyFieldForm
            from wtforms import FormField
            from page.models import Template
            template = Template.get_by_id(item_id)
            form = model_form(Template,db.session,base_class=Form,exclude=['pages','blocks','filename','body'])

            class TemplateForm(form):
                body = FormField(TemplateBodyFieldForm,separator='_')
            self._form = TemplateForm
            self._context['obj'] = template
            self._form_obj = template
        return self.render()

    def post(self,item_id=None):
        from ext import db
        from auth.models import User
        if 'content' in request.endpoint:
            session['content'] = request.form['content'][:]
            return self.redirect('admin.edit_{}'.format(request.url.split('/')[-3]),item_id='{}'.format(request.url.split('/')[-1]))
        else:
            if 'page' in request.endpoint:
                from page.models import Page as model
                needs_content = True
                msg = 'cms page'
                redirect = 'pages'
                exclude = ['added_by','date_added']
            if 'block' in request.endpoint:
                from page.models import Block as model
                needs_content = True
                msg = 'cms block'
                redirect = 'blocks'
                exclude = ['templates','pages']
            if 'template' in request.endpoint:
                from page.models import Template as model
                needs_content = False
                msg = 'template'
                redirect = 'templates'
                exclude = ['pages','blocks']
            self._form = model_form(model,db.session,base_class=Form,exclude=exclude)(request.form)
            obj = model.query.filter(model.id==item_id).first()
            self._form.populate_obj(obj)
            if needs_content:
                if 'content' in session:
                    obj.content = session.pop('content',None)
            obj.update()
            flash('Successfully updated {}: {}'.format(msg,obj.name))
            return self.redirect('admin.{}'.format(redirect))

class TestView(BaseView):
    _template = 'add.html'
    _form = None
    _context = {}

    def get(self):
        from admin.forms import TestForm
        self._form = TestForm
        self._form._has_pagedown = False
        self._context['url_link'] = 'admin.test'
        return self.render()



class PageListView(BaseView):
    _template = 'list_pages.html'
    _context = {}

    def get(self):
        from page.models import Page
        pages = [FakePage(page) for page in Page.query.all()]
        self._context['page_list'] = pages
        return self.render()


class AdminSettingsView(BaseView):
    _template = 'settings.html'
    _form = None # uses AddSettingForm and SettingForm
    _context = {}

    def get(self):
        self._get_settings_widgets()
        return self.render()

    def _get_settings_widgets(self):
        from .models import Setting
        self._context['widgets'] = [x.widget for x in Setting.query.all()]


class AdminAddCategoryView(BaseView):
    _template = 'add.html'
    _form = None
    _context = {}
    _form_heading = 'Add a Category'
   
    decorators = [login_required]

    def get(self):
        self._context['form_args'] = {'heading':self._form_heading}
        return self.render()

    def post(self):
        self._context['form_args'] = {'heading':self._form_heading}
        self._form = self._form(request.form)
        if self._form.validate():
            class Category(object):
                def save(self):
                    pass
            c = None#Category.query.filter(Category.name==self._form.name.data).first()
            if c is None:
                c = Category()
                c.name = self._form.name.data
                c.description = self._form.description.data
                #c.save()
                self.flash('You added category: {}'.format(c.name),'success')
            else:
                self.flash('There is already a category by that name, try again','danger')
        return self.render()

class AdminBlogView(BaseView):
    _template = 'blog_settings.html'
    _form = None
    _context = {}

    def get(self,blog_id=None):
        return self.render()

class AdminAddBlogView(BaseView):
    _template = 'add.html'
    _form = None
    _context = {}

    decorators = [login_required]

    def get(self):
        self._form = AddBlogForm(author_id=session.get('user_id'),date_added=datetime.now())
        return self.render()

    def post(self):
        self._form = AddBlogForm(request.form)
        if self._form.validate():
            class Blog(object):
                def save(self):
                    pass
            blog = Blog()
            blog.name = self._form.name.data
            blog.date_added = self._form.date_added.data
            blog.title = self._form.title.data
            blog.slug = self._form.slug.data
            author_id = self._form.author_id.data
            category = self._form.category.data
            blog.save()
        return self.redirect('blog.list')





    
from flask import jsonify
@admin.route('/add/tst')
def tst():
    num = request.args.get('num',None)
    if num is None:
        res = {'val':'none'}
    else:
        res = {'num':num*2}
    return jsonify(result=res)






# define bluepriont specific context processors

def example_context():
    return {'example':'value'}

'''def nav():
    brand = 'Admin'
    nav_links = {
            'Home':'index',
            'Files':'index',
            'Other':'index'
            }

    dropdowns = {
            'Dropdown One':{
                'nothing':'index',
                'more':'index',
                'sep':None,
                'more nothing':'admin'
            }
        }
'''
def frontend_nav():
    return {'nav_links':
            (
                ('core.index','Home'),
                ('blog.index','Blog'),
                ('core.meet','Meet Us'),
                ('core.about','About'),
                ('core.contact','Contact'),
            ),
            'nav_title':'My Site',
    }

def admin_nav():
    return {'admin_nav_links':
            (
                ('DashBoard','admin.index'),
            ),
            'admin_nav_title':'Admin',
            'admin_dropdowns':(
                dict(
                    Manage=dict(
                                    Users='admin.users',
                                    Blogs='admin.blogs',
                                    Settings='admin.settings',
                              )
                    ),
                dict(
                    List=dict(
                                    Blocks='admin.blocks',
                                    Pages='admin.pages',
                                    Templates='admin.templates',
                              )
                    ),
                dict(
                    Add=dict(
                                    Template='admin.add_template',
                                    Page='page.add_page',
                                    Block='admin.add_block',
                                    Blog='admin.add_blog',
                              )
                    ),
                ),
        }




# frontend nav title


#def admin_title():
#    return {'admin_title':'Admin'}
# register this in settings.py to make it avialible in all templates



import os
import json
from flask import current_app
from flask_admin import Admin, BaseView,expose,helpers,AdminIndexView
from flask_admin.contrib.sqla.view import ModelView
from .models import Page, Category
from .forms import ContactUsSettingsForm
from . import page




class IndexView(AdminIndexView):
    @expose()
    def index(self,*args,**kwargs):
        return self.render('admin/dashboard.html')

admin = Admin(name='Page Admin',index_view=IndexView(),template_mode='bootstrap3')

@page.before_app_first_request
def add_admin():
    current_app.test_request_context().push()
    admin.init_app(current_app)
    current_app.test_request_context().pop()

class PageAdmin(ModelView):
    column_list = ['title','slug','template_file','date_modified']

    def __init__(self,*args,**kwargs):
        ModelView.__init__(self,model=Page,session=Page._session,*args,**kwargs)

class ContactUsAdmin(BaseView):
    @expose('/',methods=['post','get'])
    def index(self):
        form = ContactUsSettingsForm(flask.request.form)
        if flask.request.method.lower() == 'post':
            data = dict(
                address=form.address.data,
                email=form.email.data,
                phone=form.phone.data,
                hours=form.hours.data,
                facebook_link=form.facebook_link.data,
                twitter_link=form.twitter_link.data,
                google_link=form.google_link.data,
            )
            with open(os.path.join(base,'contact_data.json'),'w') as f:
                f.write(json.dumps(data))
        data = json.loads(open(os.path.join(base,'contact_data.json'),'r').read())
        form = ContactUsSettingsForm(**data)
        return self.render('admin/settings.html',form=form)

class CategoryAdmin(ModelView):
    def __init__(self,*args,**kwargs):
        ModelView.__init__(self,model=Category,session=Category._session,*args,**kwargs)

for category,_admin in [
            ('settings',ContactUsAdmin),
            ('cms',PageAdmin),
            ('cms',CategoryAdmin),
    ]:
    admin.add_view(
        _admin(
            category = category
        )
    )



from mrbob import bobexceptions as mbe

# post question hooks - takes (configurator,question,answer)
def skip_next_if(c,q,a):
    if a:
        c.questions.pop(c.questions.index(q)+1)
    return a

def skip_next_if_not(c,q,a):
    if not a:
        c.questions.pop(c.questions.index(q)+1)
    return a

def to_list(c,q,a):
    return a.split(',')

def choose_app_template(c):
    if c.variables['app.language'].startswith('c'):
        c.template_dir = '{}_coffee'.format(c.template_dir)

# pre question hooks - takes (configurator,question)
def only_if_type(c,q):
    print c.variables['service.service_type']
    print q.extra.get('check_type')
    if c.variables['service.service_type'] != q.extra.get('check_type'):
        raise mbe.SkipQuestion

def add_apps(c,q):
    if not c.variables['project.add_apps']:
        raise mbe.SkipQuestion

def check_for_captcha(c,q):
    if not c.variables['local_settings.use_captcha']:
        raise mbe.SkipQuestion

def db_related_skip(c,q):
    if not c.variables['project.use_database']:
        raise mbe.SkipQuestion

# pre render hooks - takes nothing - maybe configurator

def get_service_type(c):
    c.template_dir = '{}/{}'.format(c.template_dir,c.variables['service.service_type'])    

def make_db_uri(c):
    if c.variables['project.use_database']:
        fmt = '%s://%s:%s@%s:%d/%s' 
        c.variables['project.db_uri'] = fmt % (c.variables['project.db-driver'],
                                               c.variables['project.db-username'],
                                               c.variables['project.db-password'],
                                               c.variables['project.db-host'],
                                               c.variables['project.db-port'],
                                               c.variables['project.db-name'])
# post render hooks - takes (configurator)






#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    manage
    ~~~~~~
"""
import subprocess
from flask.ext.script import Shell, Manager, prompt_bool
from flask.ext.script.commands import Clean,ShowUrls
from flask.ext.xxl import flaskxxl
manager = Manager(flaskxxl)
from flask.ext.xxl.mr_bob import manager as mrbob_manager
import sys

def main(auto=False):
    if not auto:
        sys.argv.insert(1,'mrbob')
    flaskxxl.test_request_context().push() 
    manager.add_command('mrbob',mrbob_manager)
    manager.add_command('clean',Clean())
    manager.add_command('urls',ShowUrls())
    default_cmd = 'mrbob'
    if len(sys.argv) > 2:
        default_cmd = 'clean'
    manager.run(default_command=default_cmd)

if __name__ == '__main__':
    main(auto=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    manage
    ~~~~~~
"""
import subprocess
from flask.ext.script import Shell, Manager, prompt_bool
from flask.ext.script.commands import Clean,ShowUrls
from flask.ext.xxl import flaskxxl
manager = Manager(flaskxxl)
from flask.ext.xxl.mr_bob import manager as mrbob_manager

def main():
    flaskxxl.test_request_context().push() 
    manager.add_command('mrbob',mrbob_manager)
    manager.add_command('clean',Clean())
    manager.add_command('urls',ShowUrls())
    manager.run(default_command='mrbob')

if __name__ == '__main__':
    main()



from flask import session,redirect,request,render_template
from flask_xxl.baseviews import BaseView
from . import auth
from .forms import UserLoginForm, UserSignupForm
from .utils import login_user, logout_user, login_required, admin_required
from sqlalchemy.exc import IntegrityError


class AuthLoginView(BaseView):
    _template = 'auth_login.html'
    _form = UserLoginForm
    _context = {}

    def get(self):        
        return self.render()

    def post(self):
        form = self._form()
        from .models import User
        if form.validate():
            email = form.email.data
            pw = form.password.data
            remember = form.keep_me_logged_in.data
            u = User.get_by_email(email)
            if u is not None:
                if u.check_password(pw):
                    login_user(u)
                    if 'next' in request.args:
                        return redirect(request.args['next'])
                    else:
                        return self.redirect('core.index')
                else:
                    self.flash('incorrect password')
            else:
                self.flash('user does not exist')
                return self.redirect('auth.signup')
        return self.render()

class AuthSignupView(BaseView):
    _template = 'register.html'
    _form = UserSignupForm
    _context = {}
    
    def get(self):
        return self.render()

    def post(self):
        from auth.models import User
        form = self._form()
        if form.validate():
            email = form.email.data
            pw = form.password.data
            u = User(email=email)
            u.password = pw
            login_user(u)
            self.flash("Thank you for signing up {}".format(email))
            return self.redirect('core.index')
        return self.redirect('auth.signup')


class AuthLogoutView(BaseView):
 
    def get(self):
        from .models import User
        if 'user_id' in session:
            user = User.get_by_id(session['user_id'])
            logout_user(user)
        return self.redirect('core.index')


@auth.app_errorhandler(404)
def not_found(e):
    return render_template('error.html',error_code=404)




# -*- coding: utf-8 -*-
"""
    basemodels.py
    ~~~~~~~~~~~
"""
import os
from functools import wraps
from werkzeug.local import LocalProxy
from flask import current_app,g
try:
    from flask import _app_ctx_stack as stack
except ImportError:
    from flask import _request_ctx_stack as stack

from inflection import underscore, pluralize
from sqlalchemy.ext.declarative import as_declarative, declarative_base, declared_attr
from sqlalchemy.ext.declarative.api import DeclarativeMeta
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy import UniqueConstraint,Column,Integer,Text,String,Date,DateTime,ForeignKey,func,create_engine

# classproperty decorator
class classproperty(object):
    def __init__(self, getter):
        self.getter = getter
    def __get__(self, instance, owner):
        return self.getter(owner)


echo_sql = lambda: os.environ.get('DATABASE_URI') and current_app.config.get('SQLALCHEMY_ECHO',False)
get_engine = lambda: create_engine(os.environ.get('DATABASE_URI') or current_app.config['DATABASE_URI'],echo=echo_sql())

Base = declarative_base()


class SQLAlchemyMissingException(Exception):
    pass

class BaseMixin(Base):
    __table_args__ = {
        'extend_existing':True
    }
    __abstract__ = True
    _session = None
    _engine = None
    _query = None
    _meta = None

    def __init__(self,*args,**kwargs):
        super(BaseMixin,self).__init__(*args,**kwargs)
        metadata = BaseMixin.metadata or Base.metadata
        self.metadata = BaseMixin.metadata = metadata

    @classproperty
    def engine(cls):
        if BaseMixin._engine is None:
           BaseMixin._engine = get_engine()
        BaseMixin._engine.echo = echo_sql()
        return BaseMixin._engine

    @declared_attr
    def id(self):
        return Column(Integer,primary_key=True)

    @classproperty
    def session(cls):
        if BaseMixin._session is None:
            BaseMixin._session = scoped_session(sessionmaker(bind=cls.engine))()
        return BaseMixin._session

    @classproperty
    def query(cls):
        if cls._query is None:
            cls._query = cls.session.query(cls)
        return cls._query

    @declared_attr
    def __tablename__(self):
        return underscore(pluralize(self.__name__))

    @classmethod
    def get_by_id(cls, id):
        if any(
            (isinstance(id, basestring) and id.isdigit(),
             isinstance(id, (int, float))),
        ):
            return cls.query.get(int(id))
        return None

    @classmethod
    def get_all(cls):
        return cls.query.all()

    @classmethod
    def create(cls, **kwargs):
        instance = cls(**kwargs)
        return instance.save()

    def update(self, commit=True, **kwargs):
        for attr, value in kwargs.iteritems():
            setattr(self, attr, value)
        return commit and self.save() or self

    def save(self,commit=True,testing=False):
        try:
            self.session.add(self)
            if commit:
                self.session.commit()
        except Exception, e:
            if not testing:
                raise e
            print e.message
            return False
        return self

    def delete(self, commit=True):
        self.session.delete(self)
        return commit and self.session.commit()

    @property
    def absolute_url(self):
        return self._get_absolute_url()

    def _get_absolute_url(self):
        raise NotImplementedError('need to define _get_absolute_url')

    @classmethod
    def get_all_columns(cls,exclude=['id']):
        if not 'id' in exclude:
            exclude.append('id')
        rtn = []
        for col in cls.__table__.c._all_columns:
            if not col.name in exclude and not col.name.endswith('id'):
                rtn.append((col.name,_clean_name(col.name)))
        for attr in dir(cls):
            if not attr in exclude:
                if not attr in [x[0] for x in rtn]:
                    if not attr.startswith('_') and not attr.endswith('id'):
                        if not callable(getattr(cls,attr)):
                            rtn.append((attr,_clean_name(attr)))
        return rtn


class AuditMixin(object):
    __abstract__ = True

    @declared_attr
    def date_added(self):
        return sq.Column(sq.DateTime,default=dt.now)

    @declared_attr
    def date_modified(self):
        return sq.Column(sq.DateTime,onupdate=dt.now)

def _clean_name(name):
    names = name.split('_')
    if len(names) > 1:
        if len(names) == 2:
            name = names[0].title() + ' ' + names[-1].title()
        else:
            name = names[0].title() + ' {} '.format(' '.join(map(str,names[1:-1]))) + names[-1].title()
    else:
        name = names[0].title()
    return name

class DBObject(object):
    _session = None
    _engine = None
    _metadata = None

    @staticmethod
    def check_ctx():
        return stack.top is not None

    @classproperty
    def session(cls):
        sess = cls._session
        if sess is None:
            sess = DBObject._session = scoped_session(sessionmaker(bind=DBObject.engine))
        return sess

    @classproperty
    def engine(cls):
        engine = cls._engine
        if engine is None:
            engine = cls._engine = cls.check_ctx() and create_engine(current_app.config.get('DATABASE_URI') or os.environ.get('DATABASE_URI'))
        return engine

    @classproperty
    def metadata(cls):
        meta = cls._metadata
        if meta is None:
            meta = cls._metadata = BaseMixin.metadata
        meta.bind = meta.bind if meta.bind else cls.engine
        return meta

db = LocalProxy(DBObject)



from flask.ext.script import Manager, prompt_bool
import os
from mrbob import cli
import mrbob
from jinja2.loaders import FileSystemLoader
import sys


manager = Manager(usage="performs mr.bob template runs")

EXT_ROOT = os.path.abspath(os.path.dirname(__file__))

MRBOB_TEMPLATE_DIR = os.path.join(EXT_ROOT,'templates')
mrbob.rendering.jinja2_env.loader = FileSystemLoader(MRBOB_TEMPLATE_DIR)


def get_all_files(dirname,seen=None):
    rtn = set()
    if seen is None:
        seen = set()
    seen.add(dirname)
    for dname,dlist,flist in os.walk(dirname):
        for itm in flist:
            if not itm.endswith('.pyc'):
                rtn.add(os.path.join(dname,itm))
        if len(dlist) == 0:
            return rtn
        else:
            for d in dlist:
                pth = os.path.join(dname,d)
                if not pth in seen:
                    rtn.update(get_all_files(pth,seen))

remove_dir = lambda dirname,path: path.split(dirname)[-1]
remove_bob = lambda x: x.replace('.bob','')


def get_templates():
    return [name for name in os.listdir(MRBOB_TEMPLATE_DIR) if not name.startswith('_') and not name.endswith('.pyc')]

@manager.command
def print_template_dir():
    print MRBOB_TEMPLATE_DIR

@manager.command
def print_templates():
    print ', '.join(map(str,get_templates()))

@manager.option('template',nargs='?',default=None)
def print_template_files(template=None):
    if template is None:
        print 'please provide a template name'
        return
    else:
        if not template in get_templates():
            print 'Error! {} is not installed'.format(template)
            return
    print '\n'.join(
                    map(
                        str,
                            [remove_bob(remove_dir(
                                MRBOB_TEMPLATE_DIR,x
                        )) for x in get_all_files(
                                            os.path.join(
                                                    MRBOB_TEMPLATE_DIR,template
                                            )
                        ) if not x is None and not x.endswith('.pyc')
                             ]
                        )
    )
_template_dir = lambda name: os.path.join(MRBOB_TEMPLATE_DIR,name)
    
@manager.option("-t","--testing",action='store_true')
def start_project(testing=False):
    template_dir = _template_dir('project')
    if testing:
        target_dir = './testing'
    else:
        target_dir = os.curdir
    if target_dir == os.curdir:
        if not prompt_bool('not testing, are you sure you want to continue...'):
            sys.exit(0)
    list_questions = False
    non_interactive = False
    args = [template_dir,'-O',target_dir,'-v','-w']
    cli.main(args)

@manager.option("-t","--testing",action="store_true")
def add_blueprint(testing=False):
    template_dir = _template_dir('blueprint')
    if testing:
        target_dir = './testing'
    else:
        target_dir = os.curdir
    if target_dir == os.curdir:
        if not prompt_bool('not testing, are you sure you want to continue...'):
            sys.exit(0)
    list_questions = False
    non_interactive = False
    args = [template_dir,'-O',target_dir,'-v','-w']
    cli.main(args)

@manager.command
def angular_app():
    template_dir = _template_dir('angular_app')
    target_dir = os.path.join(os.curdir,'static','js')
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)
    list_questions = False
    non_interactive = False
    args = [template_dir,'-O',target_dir,'-v','-w']
    cli.main(args)

@manager.command
def angular_service():
    template_dir = _template_dir('angular_service')
    target_dir = os.path.join(os.curdir,'static','js')
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)
    list_questions = False
    non_interactive = False
    args = [template_dir,'-O',target_dir,'-v','-w']
    cli.main(args)








import sqlalchemy as sa
from ...basemodels import BaseMixin
from LoginUtils import check_password, encrypt_password
from ...baseviews import is_verbose


if is_verbose():
    print 'importing flask_xxl.apps.admin.models as ',__name__


class Setting(BaseMixin):

    name = sa.Column(sa.String(255),nullable=False,unique=True)
    setting_type_id = sa.Column(sa.Integer,sa.ForeignKey('types.id'))
    setting_type = sa.orm.relationship('Type',backref=sa.orm.backref(
        'settings',lazy='dynamic'))
    default = sa.Column(sa.String(255))
    value = sa.Column(sa.String(255))

    @property
    def widget(self):
        if self.type:
            return self.type.widgets
        else:
            return ''

class Type(BaseMixin):

    name = sa.Column(sa.String(255),nullable=False)
    widgets = sa.orm.relationship('Widget',backref=sa.orm.backref(
        'type'),lazy='dynamic')
    html = sa.Column(sa.Text)
    field_type = sa.Column(sa.String(255))
    required = sa.Column(sa.Boolean,default=False)
    data_type = sa.Column(sa.String(255))

    def __repr__(self):
        return self.name or ''

class Widget(BaseMixin):

    name = sa.Column(sa.String(255),nullable=False)
    title = sa.Column(sa.String(255))
    content = sa.Column(sa.Text,nullable=False)
    type_id = sa.Column(sa.Integer,sa.ForeignKey('types.id'))

    def __repr__(self):
        return self.name




import flask
import json
import os
import sqlalchemy as sq
from sqlalchemy import orm
sa = sq
import datetime as dt
from flask_xxl.basemodels import AuditMixin,BaseMixin as Model
#try:
#    from ext import db as Model
#except ImportError:
#    import sys
#    print 'the flask blueprint flask_xxl.apps.page needs a database configured as db in the ext module to work correctley'
#    sys.exit(1)

app_base_dir = os.path.abspath(os.path.dirname(__file__))

class Template(Model):
    name = sq.Column(sq.String(255))

class Block(Model):
    name = sq.Column(sq.String(255))

class Page(Model):

    __tablename__ = 'pages'

    DEFAULT_TEMPLATE = 'page.html'

    id = sq.Column(sq.Integer,primary_key=True)

    parent_id = sa.Column(sa.Integer,sa.ForeignKey('pages.id'))

    children = orm.relationship('Page',backref=orm.backref(
            'parent',remote_side=[id]),lazy='dynamic',primaryjoin='Page.id==Page.parent_id',
             cascade='all,delete-orphan')
    title = sq.Column(sq.String(255),unique=True,nullable=False)
    keywords = sq.Column(sq.Text)
    slug = sq.Column(sq.String(255),unique=True,
            nullable=False)
    template_file = sq.Column(sq.String(255),
            nullable=False,default=DEFAULT_TEMPLATE)
    meta_title = sa.Column(sa.String(255))
    add_right_sidebar = sq.Column(sq.Boolean,default=False)
    add_left_sidebar = sq.Column(sq.Boolean,default=False)
    add_to_nav = sq.Column(sq.Boolean,default=False)
    body_content = sq.Column(sq.Text)

    _current = False

    @property
    def has_children(self):
        return bool(any(self.children))

    def is_parent_to(self,page=False):
        if page:
            return page.id == self.parent.id
        return self.has_children

    def is_child_of(self,page=None):
        if page:
            return page in self.children
        return self.parent is not None

    def add_child(self,page):
        # dont reference any page twice
        if not page in self.children:
            # dont refrence parent
            if self.parent and self.parent.id != page.id:
                self.children.append(page)
                self.save()

    @property
    def navlink(self):
        return (self.title,self.get_absolute_url())

    def get_absolute_url(self):
        return flask.url_for('.page',slug=self.slug)

    @classmethod
    def get_by_slug(cls,slug):
        return cls.query().filter(Page.slug==slug).first()

    @classmethod
    def get_page_count(cls):
        return cls.query().count()

    def __repr__(self):
        return '<{}: #{}-{}'.format(self.__class__.__name__,self.id,self.slug)

    def __str__(self):
        return unicode(self)

    def __unicode__(self):
        return self.title


class Category(Model):

    name = sa.Column(sa.String(255),nullable=False,unique=True)
    description = sa.Column(sa.Text)

    def __unicode__(self):
        return self.name



import os
import json
from flask.views import MethodView
from flask.templating import render_template
from flask.helpers import url_for
from flask import redirect, flash, request
from inflection import pluralize
from wtforms.form import FormMeta
from .basemodels import classproperty


is_verbose = lambda: os.environ.get('VERBOSE') and True or False

class Flasher(object):
    DEFAULT_CATEGORY = 'warning'
    WARNING_CLASS = 'warning'
    INFO_CLASS = 'info'
    SUCCESS_CLASS = 'success'
    ERROR_CLASS = 'danger'

    def __init__(self,default=None,class_map=None):
        if default:
            self.DEFAULT_CATEGORY = default
        if class_map:
            self._set_classes(class_map)

    def _set_classes(self,class_map):
        default_map = {
            'success':self.SUCCESS_CLASS,
            'warning':self.WARNING_CLASS,
            'error':self.ERROR_CLASS,
            'info':self.INFO_CLASS
        }
        for k in class_map:
            tmp = class_map.get(k)
            if tmp is not None:
                default_map[k] = tmp


    def flash(self,msg,cat=None):
        cat = cat or self.DEFAULT_CATEGORY
        return flash(msg,cat)

    def add_warning(self,msg):
        return self.flash(msg,self.WARNING_CLASS)

    def add_error(self,msg):
        return self.flash(msg,self.ERROR_CLASS)

    def add_info(self,msg):
        return self.flash(msg,self.INFO_CLASS)

    def add_success(self,msg):
        return self.flash(msg,self.SUCCESS_CLASS)


class PostViewAddon(object):
    def _process_post(self):
        self.post_data = ((request.data and json.loads(request.data)) if not request.form else dict(request.form.items())) if not request.mimetype == 'application/json' else request.json


class BaseView(MethodView):
    _template = None
    _form = None
    _context = {}
    _form_obj = None
    _obj_id = None
    _form_args = {}
    _default_view_routes = {}
    _flasher = None
    _flasher_class_map = None

    def __init__(self,*args,**kwargs):
        super(BaseView,self).__init__(*args,**kwargs)
        if self._flasher_class_map is not None:
            self._flasher = Flasher(class_map=self._flasher_class_map)
        else:
            self._flasher = Flasher()


    def flash(self,msg):
        return self.flasher.flash(msg)

    def success(self,msg):
        return self._flasher.add_success(msg)

    def warning(self,msg):
        return self._flasher.add_warning(msg)

    def info(self,msg):
        return self._flasher.add_info(msg)

    def error(self,msg):
        return self._flasher.add_danger(msg)

    @classmethod
    def _add_default_routes(cls,app=None):        
        for route,endpoint in cls._default_view_routes.items():
            if is_verbose():
                print 'attaching',route,'to view func',endpoint
            app.add_url_rule(route,endpoint,view_func=cls.as_view(endpoint))

    def render(self,**kwargs):
        if self._template is None:
            return NotImplemented
        if kwargs:
            self._context.update(kwargs)
        if self._form is not None:

            if type(self._form) == FormMeta:
                if self._form_obj is not None:
                    self._context['form'] = self._form(obj=self._form_obj,**self._form_args)
                else:
                    self._context['form'] = self._form(**self._form_args)
                if self._obj_id is not None:
                    self._context['obj_id'] = self._obj_id
            else:
                self._context['form'] = self._form
            choices = self._context.get('choices')
            if choices:
                for field in self._context['form']:
                    if hasattr(field,field.__name__) and hasattr(getattr(field,field.__name__),field.__name__):
                        inner_field = getattr(getattr(field,field.__name__),getattr(field.__name__))
                        if hasattr(inner_field,'choices'):
                            setattr(inner_field,'choices',choices)
            for f,v in self._form_args.items():
                self._form.__dict__[f].data = v
        return render_template(self._template,**self._context)

    def redirect(self,endpoint,**kwargs):
        if not kwargs.pop('raw',False):
            return redirect(url_for(endpoint,**kwargs))
        return redirect(endpoint,**kwargs)
        
    def form_validated(self):
        if self._form:
            return self._form().validate()
        return False

    def get_form_data(self):
        result = {}
        for field in self._form():
            name = field.name
            if '_' in field.name:
                if not field.name.startswith('_'):
                    if not field.name.endswith('_'):
                        if field.name.split('_')[0] == field.name.split('_')[1]:
                            name = field.name.split('_')[0]
            result[name] = field.data
        return result
    
    def get_env(self):
        from flask import current_app
        return current_app.create_jinja_environment()
    
    @property
    def flasher(self):
        return self._flasher


class ModelView(BaseView):
    # ModelView is an abstract class
    # just an interface really
    # to use the ModelView create your own view class
    # and use this as the parent class, and 
    # set its _model class attr to the class to wrap ie:
    # 
    # class UserModelView(ModelView):
    #   _model = User
    _model = None

    def render(self,**kwargs):
        alt_model_id = '{0}_id'.format(self._model.__name__.lower())
        if self._model is not None:
            if 'model_id' in kwargs:
                model_id = kwargs.pop('model_id')
            elif alt_model_id in kwargs:
                model_id = kwargs.pop('alt_model_id')
            else:
                model_id = None
            if model_id is not None:
                self._context['object'] = self.get_by_id(model_id)
            else:
                self._context['object'] = self._model()
            self._context['model'] = self._model
        return super(ModelView,self).render(**kwargs)

    def add(self,**kwargs):
        tmp = self._model(**kwargs)
        tmp.save()
        return tmp

    def update(self,model_id,**kwargs):
        tmp = self._model.query.filter_by(self._model.id==model_id).first()
        if 'return' in kwargs:
            if kwargs.pop('return',None):
                rtn = True
            else:
                rtn = False
        for k in kwargs.keys():
            tmp.__dict__[k] = kwargs[k]
        tmp.save()
        if rtn: return tmp

    def get_all(self):
        return self._model.get_all()

    def get_by_id(self,model_id):
        return self._model.get_by_id(model_id)

class AddModelView(ModelView,PostViewAddon):
    _success_endpoint = None
    _success_message = 'You successfully added an item'
    
    def get(self):
        return self.render()

    def post(self):
        self._process_post()
        self._context['obj'] = self._model(**self.post_data).save()
        self.success(self._success_message)
        return self.redirect(self._success_endpoint or '.index')


def AddModelApiView(ModelView,PostViewAddon):
    def post(self):
        self._process_post()
        return jsonify(**self.add(**self.post_data).to_json())

class ListModelView(ModelView):
    def get(self):
        name = pluralize(self._model.__name__)
        return jsonify(name=[m.to_json() for m in self.get_all()])

class ViewModelView(ModelView):
    def get(self,item_id):
        m = self.get_by_id(item_id)
        return jsonify(**m.to_json())

class ModelAPIView(ModelView):
    __abstract__ = True

    @classproperty
    def _default_view_routes(cls):
        if cls is ModelAPIView:
            return {}
        name = cls.__name__.lower()
        _default_view_routes = {
                '/{}/list'.format(name):'{}-list'.format(name),
                '/{}/detail'.format(name):'{}-detail'.format(name),
                '/{}/edit'.format(name):'{}-edit'.format(name),
        }
        return _default_view_routes


    def render(self,**kwargs):
        old_rtn = super(ModelAPIView,self).render(**kwargs)
        rtn = make_response(json.dumps(self._context))
        rtn.headers['Content-Type'] = 'application/json'
        return rtn
    
    



from flask.ext.script import prompt_bool
from manage import manager
from mrbob import cli
import sys
import os
from flask_xxl import flaskxxl



def get_template_dir():
    EXT_ROOT = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(EXT_ROOT,'templates')


def print_usage():
    print 'flaskxxl-manage.py [start-project] [start-blueprint]'

def run_mrbob(template_dir,testing):
    if testing:
        target_dir = './testing'
    else:
        target_dir = os.curdir
    if target_dir == os.curdir:
        if not prompt_bool('not testing, are you sure you want to continue...'):
            sys.exit(0)
    list_questions = False
    non_interactive = False
    args = [template_dir,'-O',target_dir,'-v','-w']
    cli.main(args)


@manager.command
def start_blueprint(testing=False):
    template_dir = os.path.join(get_template_dir(),'blueprint')
    run_mrbob(template_dir,testing)




@manager.command
def start_project(testing=False):
    template_dir = os.path.join(get_template_dir(),'project')
    run_mrbob(template_dir,testing)


def main():
    func = None
    testing = False
    if len(sys.argv) <= 1:
        print_usage()
    else:
        if sys.argv[1] == 'start-project':
            func = start_project
        elif sys.argv[1] == 'start-blueprint':
            func = start_blueprint
        else:
            print_usage()
            sys.exit()
        if len(sys.argv) > 2:
            if sys.argv[2] == '-t' or sys.argv[2] == '--testing':
                testing = True
            else:
                testing = False
        if func is not None:
            func(testing)

if __name__ == "__main__":
    with flaskxxl.context() as ctx:
        manager.run()



import datetime
import math

__all__ = ['human_time','date','date_pretty','datetime','pluralize','month_name']

def datetimeformat(value):
    delta = datetime.datetime.now() - value
    if delta.days == 0:
        formatting = 'today'
    elif delta.days < 10:
        formatting = '{0} days ago'.format(delta.days)
    elif delta.days < 20:
        formatting = '{0} weeks ago'.format(int(math.ceil(delta.days/7.0)))
    elif value.year == datetime.datetime.now().year:
        formatting = 'this year, on %d %b'
    else:
        formatting = 'on %b %d %Y'
    return value.strftime(formatting)

def parse_date(date):
    import dateutil.parser as p
    return p.parse(str(date))


def human_time(timestamp):
    d = parse_date(timestamp)
    return datetimeformat(datetime.datetime(d.year,d.month,d.day))

def date(value):
    """Formats datetime object to a yyyy-mm-dd string."""
    value = parse_date(value)
    return value.strftime('%Y-%m-%d')

def date_time_pretty(value):
    """Formats datetime object to a Month dd, yyyy string."""
    value = parse_date(value)
    return value.strftime('%B %d, %Y at %I:%M %P')

def date_pretty(value):
    """Formats datetime object to a Month dd, yyyy string."""
    value = parse_date(value)
    return value.strftime('%h %d, %Y' )


def dt(value):
    """Formats datetime object to a mm-dd-yyyy hh:mm string."""
    value = parse_date(value)
    return value.strftime('%m-%d-%Y %H:%M')


def pluralize(value, one='', many='s'):
    """Returns the plural suffix when needed."""
    value = parse_date(value)
    return one if abs(value) == 1 else many


def month_name(value):
    """Return month name for a month number."""
    value = parse_date(value)
    from calendar import month_name
    return month_name[value.month]

def get_day(value):
    value = parse_date(value)
    return value.day



def split(value,symbol=' '):
    return value.split(symbol)



from basewidgets import ModelWidget
from blog import models

class TagWidget(ModelWidget):
    _model = models.Tag

    def render(self,*args,**kwargs):
        rtn = '<ul>\n'
        for tag in self._model.query.all():
            rtn += '<li class=tag><a href="{}">{}</a></li>\n'.format(tag.link,tag.name)
        return rtn + '</ul>'






from . import auth 
from .views import AuthLogoutView,AuthLoginView,AuthSignupView

routes = [
        ((auth),
            ('/logout',AuthLogoutView.as_view('logout')),
            ('/','admin_list',AuthLoginView),
            ('/login','login',AuthLoginView),
            ('/register','signup',AuthSignupView),
        )
    ]



from flask import Markup
try:
    from markdown2 import markdown as md2
except ImportError:
    from markdown import markdown as md2
# Jinja

def date(value):
    """Formats datetime object to a yyyy-mm-dd string."""
    return value.strftime('%Y-%m-%d')


def date_pretty(value):
    """Formats datetime object to a Month dd, yyyy string."""
    return value.strftime('%B %d, %Y')


def datetime(value):
    """Formats datetime object to a mm-dd-yyyy hh:mm string."""
    return value.strftime('%m-%d-%Y %H:%M')


def pluralize(value, one='', many='s'):
    """Returns the plural suffix when needed."""
    return one if abs(value) == 1 else many


def month_name(value):
    """Return month name for a month number."""
    from calendar import month_name
    return month_name[value]


def markdown(value):
    """Convert plain text to HTML."""
    extras = ['fenced-code-blocks', 'wiki-tables','attr_list',
              'fenced-code','def-list','tables','extras','meta',
              'nl2br','smart_lists','toc','markdown_checklist.extension']
    return Markup(md2(value, extras=extras))




# -*- coding: utf-8 -*-
"""
    testing.py
    ~~~~~~~
    TestCase for testing
    :license: BSD3
"""
from flask.ext.testing import TestCase
from .main import AppFactory
from .basemodels import BaseMixin as BaseModel
from settings import TestingConfig
from sqlalchemy import MetaData

meta = BaseModel.metadata


class BaseTestCase(TestCase):

    def create_app(self):
        return AppFactory(TestingConfig).get_app(__name__)

    def setUp(self):
        self.app = self.create_app()
        self.db = BaseModel
        import sqlalchemy_utils as squ
        if squ.database_exists(self.db.engine.url):
            squ.drop_database(self.db.engine.url)
        squ.create_database(self.db.engine.url)
        #import imports
        #for module in dir(imports):
        #    globals()[module] = getattr(imports,module)            
        meta.bind = self.db.engine
        meta.create_all()

    def tearDown(self):
        self.db.session.close()
        meta.drop_all()


    def assertContains(self, response, text, count=None,
                       status_code=200, msg_prefix=''):
        """
        Asserts that a response indicates that some content was retrieved
        successfully, (i.e., the HTTP status code was as expected), and that
        ``text`` occurs ``count`` times in the content of the response.
        If ``count`` is None, the count doesn't matter - the assertion is true
        if the text occurs at least once in the response.
        """

        if msg_prefix:
            msg_prefix += ": "

        self.assertEqual(response.status_code, status_code,
            msg_prefix + "Couldn't retrieve content: Response code was %d"
                         " (expected %d)" % (response.status_code, status_code))

        real_count = response.data.count(text)
        if count is not None:
            self.assertEqual(real_count, count,
                msg_prefix + "Found %d instances of '%s' in response"
                             " (expected %d)" % (real_count, text, count))
        else:
            self.assertTrue(real_count != 0,
                    msg_prefix + "Couldn't find '%s' in response" % text)



from flask import session, g, flash, redirect, url_for, request
from functools import wraps


def login_user(user):
    session['logged_in'] = True
    session['user_id'] = user.id
    session['email'] = user.email

def logout_user(user):
    session.pop('logged_in',None)
    session.pop('user_id',None)
    session.pop('email',None)

def login_required(view):
    @wraps(view)
    def wrapper(*args, **kwargs):
        if 'logged_in' in session:
            return view(*args, **kwargs)
        else:
            flash('You need to login first.')
            return redirect(url_for('auth.login',next=request.url))
    return wrapper

def admin_required(view):
    @wraps(view)
    def wrapper(*args,**kwargs):
        if 'user_id' in session:
            if User.get_by_id(session.get('user_id')).is_admin:
                return view(*args,**kwargs)
        flash('You need to login as an administrator to access that page.')
        return redirect(url_for('auth.login',next=request.url))
    return wrapper




from flask_wtf import Form
from wtforms.ext.sqlalchemy.orm import QuerySelectField
from wtforms import fields, validators,widgets
from flask_pagedown.fields import PageDownField
from flask_codemirror.fields import CodeMirrorField
from ..page.fields import CKTextEditorField
from wtforms.widgets.html5 import DateInput as DateWidget

#factory = Type.query.all()

class AddSettingForm(Form):
    name = fields.StringField('Setting Name',validators=[validators.InputRequired()])
    #type = QuerySelectField('setting type',query_factory=factory,validators=[validators.InputRequired()])
    default = fields.StringField('Default Value')
    value = fields.StringField('Setting Value')

class AddSettingTypeForm(Form):
    name = fields.StringField('Setting Type Name',validators=[validators.InputRequired()])
    widget = fields.StringField('Input Widget')

class TestForm(Form):
    title = fields.StringField('Title')
    content = PageDownField('content')

class BaseTemplateForm(Form):
    template = fields.SelectField('base template',validators=[validators.InputRequired()])#,choices=[('a','a'),('b','b'),('c','c')])

class TemplateBodyFieldForm(Form):
    body = CKTextEditorField('body')


class AddBlogForm(Form):
    name = fields.StringField('Blog Name',validators=[validators.InputRequired()])
    title = fields.StringField('Blog Title',validators=[validators.InputRequired()])
    slug = fields.StringField('Url Slug')
    author_id = fields.HiddenField()
    date_added = fields.HiddenField()
    


class AddPageForm(Form):
    date_added = fields.DateField('Publish On:',format="%m-%d-%Y",widget=DateWidget())
    date_end = fields.DateField('Expire On:',format="%m-%d-%Y",validators=[validators.Optional()],widget=DateWidget())
    name = fields.StringField('Page Name',validators=[validators.InputRequired()])
    description = fields.TextAreaField('Description',validators=[validators.Optional()])
    slug = fields.StringField('Page Slug',validators=[validators.InputRequired()])
    short_url = fields.StringField('Url',validators=[validators.Optional()])
    title = fields.StringField('Page Title',validators=[validators.InputRequired()])
    add_to_nav = fields.BooleanField('Add to Navbar')
    add_sidebar = fields.BooleanField('Add Sidebar')
    visible = fields.SelectField(choices=((1,'Publish'),(0,'Draft')))
    meta_title = fields.StringField('Meta Title',validators=[validators.InputRequired()])
    content = CodeMirrorField('Content',language='xml',config={'lineNumbers':'true'})
    template = fields.FormField(BaseTemplateForm,label="Template",separator='_')
    blocks = fields.SelectMultipleField(label="blocks",choices=[('a','a'),('b','b'),('c','c')])
    category = QuerySelectField('category')
    #tags = TagField('Tags')
    use_base_template = fields.BooleanField('Use Base Template')
    base_template =  fields.SelectField('base template',validators=[validators.InputRequired()])
    submit = fields.SubmitField('Save')



from flask import session

def user_context():
    if 'email' in session:
        if 'user_id' in session:
            from auth.models import User
            return {
                'user':User.get_by_id(session.get('user_id',None)),
                'email':session.get('email',None)
            }
    return {'user':None}
# register this in settings.py to make it avialible in all templates



from . import page
from .views import ContactFormView,PageSlugView,PagesView,AddPageView

routes = [
    (
        (page),
            ('/','page_list',PagesView),
            ('/<slug>','page',PageSlugView),
            ('/add_page','add_page',AddPageView),
            ('/contact-us','contact_us',ContactFormView),
    )
]



VERSION = '0,10,1'
import os
from setuptools import setup, find_packages,findall
from glob import glob

def get_description():
    return open('README.txt','r').read()

def get_version():
    l,m,s = VERSION.split(',')
    return '{0}.{1}.{2}'.format(l,m,s).strip()

data = os.walk(os.path.dirname(__file__))

make_file = lambda dn,f: os.path.join(os.curdir,os.sep,dn,f)

def get_pkg_data():
    data = os.walk(os.path.abspath(os.curdir))
    pkg_data = []

    for dn,dl,fl in data:
        if 'templates' in dn.split('/'):
            for f in fl:
                if not f.endswith('.py'):
                    pkg_data.append(make_file(dn,f))
    return pkg_data

config = dict(
        name='flask-xl',
        version=get_version(),#'0.0.9',
        include_package_data=True,
        author='Kyle Roux',
        author_email='kyle@level2designs.com',
        description='quick way to design large flask projects',
        long_description=get_description(),
        packages=['flask_xxl'],
        package_data = {'':findall('flask_xxl')},     #['*.bob','*.html','*.js','*.css','*',]},
        install_requires=[
            'flask==0.10.1',
            'flask-alembic==1.0.2',
            'flask-sqlalchemy==2.0',
            'flask-script==2.0.5',
            'flask-WTF==0.10.2',
            'jinja2==2.7.3',
            'LoginUtils==1.0.1',
            'Mako==1.0.0',
            'MarkupSafe==0.23',
            'SQLAlchemy==0.9.8',
            'WTForms==2.0.1',
            'Werkzeug==0.9.6',
            'alembic==0.6.7',
            'argparse==1.2.1',
            'itsdangerous==0.24',
            'wsgiref==0.1.2',
            'six==1.8.0',
            'mr.bob2==0.2.3',
            'Flask-DebugToolbar==0.9.2',
            'Flask-PageDown==0.1.5',
            'Pygments==2.0.1',
            'flask-codemirror==0.0.3',
            'jinja2-highlight==0.6.1',
            'requests==2.5.1',
            'inflection==0.2.1',
            'markdown==2.5.2',
            ],
        zip_safe=False,
        entry_points=dict(
            console_scripts='flaskxxl-manage.py=flask_xxl.manage:main'
            ),
)

if __name__ == "__main__":
    setup(**config)



from ext import db
from flask import url_for
from settings import BaseConfig

class Pagination(object):
    _model = None
    _query = None
    _page_num = 1
    _objs = []
    _obj_page = []
    _per_page = BaseConfig.ADMIN_PER_PAGE
    _page_count = _per_page

    def __init__(self,model,page_num=None,query=None):
        if query is None:
            self._query = model.query.order_by(db.desc(model.id))
        else:
            self._query = query
        self._model = model
        self._page_num = page_num or self._page_num
        self._objs = self._query.all()
        self.set_page(self._objs)

    def set_page(self,obj_list):
        s = ((self._per_page*self._page_num)-self._per_page)
        self._obj_page = []
        total = 0
        while total != self._per_page:
            for i in range(self._per_page):
                try:
                    self._obj_page.append(obj_list[s])
                except:
                    break
                finally:
                    s+=1 
                    total += 1
            break
        


    @property
    def has_next(self):
        return self._page_num != self._per_page*len(self._objs)

    @property
    def has_prev(self):
        return self._page_num != 1

    @property
    def next_link(self):
        if self.has_next:
            return url_for('admin.page_{}'.format(self._model.__name__.lower()),page_num=self.page_num+1)
        return '#'
        
    @property
    def prev_link(self):
        if self.has_prev:
            return url_for('admin.page_{}'.format(self._model.__name__.lower()),page_num=self._page_num-1)
        return '#'
        

def get_pk(obj):
    if 'id' in obj.__dict__:
        return obj.id
    raise IOError




# -*- coding: utf-8 -*-

"""
    main.py - where the magic happens
    ~~~~~~~~
    :license: BSD, see LICENSE for more details.
"""
import jinja2_highlight
import sys
import os
from flask import Flask,views
from werkzeug.utils import import_string,find_modules
import jinja2_highlight
from .baseviews import is_verbose


class MyFlask(Flask):
        jinja_options = dict(Flask.jinja_options)
        jinja_options.setdefault('extensions',
                []).append('jinja2_highlight.HighlightExtension')

class NoRouteModuleException(Exception):
    pass

class NoTemplateFilterException(Exception):
    pass

class NoContextProcessorException(Exception):
    pass

class NoBlueprintException(Exception):
    pass

class NoExtensionException(Exception):
    pass

class NoInstalledBlueprintsSettingException(Exception):
    pass

class AppFactory(object):
    _routes_registered = None

    def __init__(self, config, envvar='PROJECT_SETTINGS', bind_db_object=True):
        self.app_config = config
        self.app_envvar = os.environ.get(envvar, False)
        self.bind_db_object = bind_db_object

    def get_app(self, app_module_name, **kwargs):
        self.app = MyFlask(app_module_name, **kwargs)
        self.app.config.from_object(self.app_config)
        self.app.config.from_envvar(self.app_envvar, silent=True)
        self.app.config['VERBOSE'] = is_verbose()


        self._set_path()
        self._bind_extensions()
        #self._register_blueprints()
        self._register_routes()
        self._load_models()
        self._load_views()
        self._register_context_processors()
        self._register_template_filters()
        self._register_template_extensions()
        self._register_middlewares()

        return self.app

    def _register_middlewares(self):
        for middleware_path in self.app.config.get('MIDDLEWARE', []):
            module_name, middleware_name = middleware_path.rsplit('.', 1)
            module_name, _ = self._get_imported_stuff_by_path(middleware_path)
            middleware = getattr(module_name, middleware_name, None)
            if middleware is not None:
                self.app.wsgi_app = middleware(self.app.wsgi_app)

    def _set_path(self):
        sys.path.append(self.app.config.get('ROOT_PATH',''))

    def _is_public_attr(self,name):
        return not name.startswith('_')

    def _get_imported_stuff_by_path(self, path):
        module_name, object_name = path.rsplit('.', 1)
        module = import_string(module_name)
        return module, object_name

    def _load_resource(self,typename):
        bp_settings_path = ((self.app.config.get('BLUEPRINTS',None) and 'BLUEPRINTS') or (self.app.config.get('INSTALLED_BLUEPRINTS',None) and 'INSTALLED_BLUEPRINTS') or False)
        if not bp_settings_path:
                raise NoInstalledBlueprintsSettingException('You must have a setting for either INSTALLED_BLUEPRINTS or BLUEPRINTS')
        for blueprint_path in self.app.config.get(bp_settings_path, []):
            module_name, object_name = blueprint_path.rsplit('.', 1)
            blueprint_module, bp_name = self._get_imported_stuff_by_path(blueprint_path)
            blueprint = getattr(blueprint_module,bp_name)
            modules = find_modules(module_name)
            for module in modules:
                if typename in module:
                    mod = import_string(module)
                    for itm in dir(mod):
                        cls = getattr(mod,itm)
                        if self._is_public_attr(itm) and\
                                itm[0] == str(itm[0]).upper() and\
                                'class' in str(cls) and\
                                'view' in str(itm).lower():

                            if hasattr(cls,'_add_default_routes') and\
                                    getattr(cls,'_default_view_routes'):
                                if is_verbose():
                                    print 'getting default routes for ',cls.__name__
                                getattr(cls,'_add_default_routes')(app=blueprint or self.app)



    def _load_views(self):
        return self._load_resource('views')

    def _load_models(self):
        return self._load_resource('models')

    def _register_template_extensions(self):
        self.app.jinja_options = dict(Flask.jinja_options)
        exts = self.app.config.get('TEMPLATE_EXTENSIONS') or ['jinja2_highlight.HighlightExtension']
        self.app.jinja_options.setdefault('extensions',[])\
                                .extend(exts)

    def _bind_extensions(self):
        if self.app.config.get('VERBOSE',False):
            print 'binding extensions'
        for ext_path in self.app.config.get('EXTENSIONS', []):
            module, e_name = self._get_imported_stuff_by_path(ext_path)
            if not hasattr(module, e_name):
                raise NoExtensionException('No {e_name} extension found'.format(e_name=e_name))
            ext = getattr(module, e_name)
            if getattr(ext, 'init_app', False):
                ext.init_app(self.app)
            else:
                ext(self.app)

    def _register_template_filters(self):
        if self.app.config.get('VERBOSE',False):
            print 'registering template filters'
        for filter_path in self.app.config.get('TEMPLATE_FILTERS', []):
            module, f_name = self._get_imported_stuff_by_path(filter_path)
            if hasattr(module, f_name):
                self.app.jinja_env.filters[f_name] = getattr(module, f_name)
            else:
                raise NoTemplateFilterException('No {f_name} template filter found'.format(f_name=f_name))

    def _register_context_processors(self):
        if self.app.config.get('VERBOSE',False):
            print 'registering template context processors'
        for processor_path in self.app.config.get('CONTEXT_PROCESSORS', []):
            module, p_name = self._get_imported_stuff_by_path(processor_path)
            if hasattr(module, p_name):
                self.app.context_processor(getattr(module, p_name))
            else:
                raise NoContextProcessorException('No {cp_name} context processor found'.format(cp_name=p_name))

    def _register_blueprints(self):
        if self.app.config.get('VERBOSE',False):
            print 'registering blueprints'
        self._bp = {}
        for blueprint_path in self.app.config.get('BLUEPRINTS', []):
            module, b_name = self._get_imported_stuff_by_path(blueprint_path)
            if hasattr(module, b_name):
                #self.app.register_blueprint(getattr(module, b_name))
                self._bp[b_name] = getattr(module,b_name)
                if self.app.config.get('VERBOSE',False):
                    print 'adding {} to bp'.format(b_name)
            else:
                raise NoBlueprintException('No {bp_name} blueprint found'.format(bp_name=b_name))

    def _register_routes(self):
        if AppFactory._routes_registered is None:
            AppFactory._routes_registered = True
            if self.app.config.get('VERBOSE',False):
                print 'starting routing'
            for url_module in self.app.config.get('URL_MODULES',[]):
                if self.app.config.get('VERBOSE',False):
                    pass
                module,r_name = self._get_imported_stuff_by_path(url_module)
                if self.app.config.get('VERBOSE',False):
                    pass
                if r_name == 'routes' and hasattr(module,r_name):
                    if self.app.config.get('VERBOSE',False):
                        print '\tsetting up routing for {} with\n\troute module {}\n'.format(module.__package__,module.__name__)
                    self._setup_routes(getattr(module,r_name))
                else:
                    raise NoRouteModuleException('No {r_name} url module found'.format(r_name=r_name))
            if self.app.config.get('VERBOSE',False):
                print 'Finished registering blueprints and url routes'
        else:
            if self.app.config.get('VERBOSE',False):
                print 'skipped'

    def _check_for_registered_blueprint(self, bp):
        found = False
        for name in [str(x) for x in self.app.blueprints]:
            if bp.__class__.__name__.split('.')[-1] in name:
                found = True
        return found

    def _setup_routes(self,routes):
        for route in routes:
            blueprint,rules = route[0],route[1:]
            #for pattern,endpoint,view in rules:
            for itm in rules:
                if len(itm) == 3:
                    pattern,endpoint,view = itm
                else:
                    pattern,view = itm
                    endpoint = None
            #for pattern,endpoint,view in rules:
                if self.app.config.get('VERBOSE',False):
                    print '\t\tplugging url Pattern:',pattern
                    print '\t\tinto View class/function:',hasattr(view,'func_name') and view.view_class.__name__ or view.__name__
                    print '\t\tat endpoint:',endpoint or view.func_name
                    print
                if type(blueprint) == type(tuple()):
                    blueprint = blueprint[0]
                blueprint.add_url_rule(pattern,endpoint or view.func_name,view_func=hasattr(view,'func_name') and view or view.as_view(endpoint))
            if not self._check_for_registered_blueprint(blueprint):
                if self.app.config.get('VERBOSE',False):
                    print '\n\t\t\tNow registering {} as blueprint\n\n'.format(str(blueprint.name))
                self.app.register_blueprint(blueprint)




from flask_wtf import Form
from wtforms import fields, validators

class BaseUserForm(Form):
    email = fields.StringField('Email Address',validators=[validators.InputRequired(),
                                validators.Email()])
    password = fields.PasswordField('Password',validators=[validators.DataRequired()])

class UserLoginForm(BaseUserForm):
    keep_me_logged_in = fields.BooleanField('Keep Me Logged In')


class UserSignupForm(BaseUserForm):
    confirm = fields.PasswordField('Confirm',validators=[validators.DataRequired(),
                                validators.EqualTo('password')])







from flask.ext.codemirror.fields import CodeMirrorField
from flask.ext.pagedown.fields import PageDownField
from flask.ext.wtf import Form
from flask.ext.wtf.recaptcha.fields import RecaptchaField
from wtforms import fields,validators

from .fields import CKTextEditorField

class EditContentForm(Form):
    content = CKTextEditorField('content')


class ContactUsForm(Form):
    name = fields.StringField('Name',validators=[validators.InputRequired()])
    email = fields.StringField('Email',validators=[validators.InputRequired()])
    #subject = fields.SelectField('Subject',validators=[validators.Optional()],choices=get_choices())
    message = fields.TextAreaField('Message',validators=[validators.InputRequired()])
    recaptcha = RecaptchaField('are you a human')
    ip_address = fields.HiddenField()


class ContactUsSettingsForm(Form):
    address = fields.StringField('Business Address')
    email = fields.StringField('Contact Email')
    phone = fields.StringField('Contact Phone Number')
    hours = fields.StringField('Company Hours')
    facebook_link = fields.StringField('Facebook link')
    twitter_link = fields.StringField('twitter Link')
    google_link = fields.StringField('Google+ link')


class TestForm(Form):
    test = fields.RadioField(
            'test',
                choices = (
                    ('val','label'),
                    ('val2','label2'),
                    ('val3','label3'),
                    ('val4','label4'),
                )
    )

class AddPageForm(Form):
    pass

class FrontendEditPageForm(Form):
    title = fields.StringField('Title')
    content = PageDownField('Content')



from flask import Blueprint

auth = Blueprint('auth',__name__,
                 template_folder='templates',
                 url_prefix='/auth')
                                    


from views import *





from flask_xxl.baseviews import BaseView,ModelAPIView
import flask
from . import page
from .forms import TestForm,ContactUsForm
from .models import Page


#class TestView(ModelAPIView):
#    _model = Page

class ContactFormView(BaseView):
    _template = 'contact.html'
    _form = ContactUsForm
    _context = {}

    def get(self):
        return self.render()

    def post(self):
        return self.render()

class PageSlugView(BaseView):
    _template = 'page.html'
    _context = {}

    def get(self,slug):
        file_name = None
        if slug.endswith('.html'):
            # first try to load the html file
            if slug in page.jinja_loader.list_templates():
                file_name = slug
            else:
                slug = slug.split('.html')[0]
                try:
                    url = flask.url_for('page.'+slug)
                    return self.redirect(url)
                except:
                    pass
        test = TestForm()
        content = ''
        page_name = ''
        if '.html' in slug:
            slug = slug.split('.html')[0]
        page = Page._session.query(Page).filter(Page.slug==slug).all()
        if page is not None:
            if type(page) == list and len(page) != 0:
                page = page[0]
            content = page.body_content
            template_file = page.template_file if getattr(page,'template_file',False) and os.path.exists(page.template_file) else page.DEFAULT_TEMPLATE
            page_name = page.name
            title = page.title
            meta_title = page.meta_title
        elif file_name is not None:
            template_file = file_name
            title = slug
            meta_title = slug
        else:
            template_file = slug + '.html'
            title = slug
            meta_title = slug
        self._template = template_file
        self._context['content'] = content
        self._context['page_title'] = title
        self._context['page_name'] = page_name
        self._context['test'] = test
        return self.render()


class AddPageView(BaseView):
    pass

class PagesView(BaseView):
    pass

'''
admin_required = ''

class BasePageView(BaseView):
    _template = ''
    _context = {}
    _base_template = 'aabout.html'
    _parent_template = None

    def render(self):
        return render_template_string(self._template,**self._context)

    def get(self,slug):
        from auth.models import User
        from page.models import Page
        self._page = Page.get_by_slug(slug)
        self.frontend_nav()
        self._context['content'] = self._page.content
        if self._page.use_base_template:
            if self._page.template is not None:
                self._base_template = self._page.body
            self._process_base_template()

    def post(self,slug):
        from auth.models import User
        from page.models import Page
        self._page = Page.get_by_slug(slug)
        self.frontend_nav()
        self._context['content'] = self._page.content
        if self._page.use_base_template:
            if self._page.template is not None:
                self._base_template = self._page.body
            self._process_base_template()

    def _process_base_template(self):
        self._template = "{{template}}"
        self._template = render_template_string(self._template,template=self._base_template)
        self._template =  "{% extends '" + self._template + "' %}<br />{% block body %}{% endblock body %}<br />"

    def frontend_nav(self):
        self._setup_nav()

    def _setup_sidebar(self):
        if self._page.add_to_sidebar:
            sidebar_data = []
            pages = self._page.query.all()
            for page in pages:
                if page.add_to_sidebar:
                    sidebar_data.append(
                        ((page.name,'pages.page',dict(slug=page.slug)))
                    )
            #self._context['sidebar_data'] = ('Title',sidebar_data)

    def _setup_nav(self):
        self._context['nav_links'] = []
        pages = self._page.query.all()
        for page in pages:
            if page.add_to_nav:
                self._context['nav_links'].append(
                        (('page.pages',dict(slug=page.slug)),'{}'.format(page.name))
                )
        self._context['nav_title'] = '{}'.format(self._page.title)
        #self._setup_sidebar()


class PagesView(BasePageView):

    def get(self,slug):
        super(PagesView,self).get(slug)
        #blocks = self._page.blocks.all()
        #self._page.template.process_blocks(blocks)
        return self.render()

    def post(self,slug):
        super(PagesView,self).post(slug)
        data = request.form.get('content')
        if data != self._page.content:
            self._page.content = data
            self._page.update()
        return self.render()


class AddPageView(BaseView):

    def get(self):
        from .models import Page
        from blog.models import Tag
        data = dict(
            date_added=request.args.get('date_added',None),
            date_end=request.args.get('date_end',None),
            name=request.args.get('name',None),
            description=request.args.get('description',None),
            slug=request.args.get('slug',None),
            short_url=request.args.get('short_url',None),
            title=request.args.get('title',None),
            add_to_nav=request.args.get('add_to_nav',None),
            add_sidebar=request.args.get('add_sidebar',None),
            visible=request.args.get('visible',None),
            meta_title=request.args.get('meta_title',None),
            content=request.args.get('content',None),
            template=request.args.get('template',None),
            category=request.args.get('category',None),
            tags=request.args.get('tags',None),
            use_base_template=request.args.get('use_base_template',None),
        )
        p = Page.query.filter(Page.name==data['name']).first()
        if p is not None:
            res = 0
        else:
            tags = [x.name for x in Tag.query.all()]
            for tag in data['tags']:
                if not tag in tags:
                    t = Tag()
                    t.name = tag
                    t.save()
            p = Page()
            p.name = data.get('name')
            p.date_added = data.get('date_added')
            p.date_end = data.get('date_end',None)
            p.description = data.get('description',None)
            p.slug = data.get('slug',None)
            p.short_url = data.get('short_url',None)
            p.title = data.get('title',None)
            nav = data.get('add_to_nav',1)
            if str(nav).lower() == 'y':
                nav = 1
            else:
                nav = 0
            p.add_to_nav = nav
            sidebar = data.get('add_sidebar',0)
            if str(sidebar).lower() == 'y':
                sidebar = 1
            else:
                sidebar = 0
            p.add_sidebar = sidebar
            p.visible = data.get('visible',None)
            p.meta_title = data.get('meta_title',None)
            p.content = data.get('content',None)
            p.template = data.get('template',None)
            p.category = data.get('category',None)
            p.tags = data.get('tags',None)
            p.use_base_template = data.get('use_base_template',None)
            p.save()
            res = 1
        return jsonify(result=res,content=data['content'])
'''



from .views import (AdminDashboardView,AdminPageView,
                    AdminTemplateView,AdminBlockView,
                    AdminCMSListView,AdminListPageView,
                    AdminDetailView,AdminEditView,
                    PageListView,AdminBlogView,AdminAddCategoryView,
                    AdminAddBlogView)
from . import admin

routes = [
        ((admin),
            ('','index',AdminDashboardView),
            ('/settings','settings',AdminDashboardView),
            ('/add/category','add_category',AdminAddCategoryView),
            ('/add/page','add_page',AdminPageView),
            ('/add/blog','add_blog',AdminAddBlogView),
            ('/edit/page/content','page_content',AdminPageView),
            ('/add/template','add_template',AdminTemplateView),
            ('/add/block','add_block',AdminBlockView),
            ('/edit/block/content','block_content',AdminBlockView),
            ('/list/blocks','blocks',AdminCMSListView),
            ('/list/pages','pages',AdminCMSListView),
            ('/list/users','users',AdminCMSListView),
            ('/list/templates','templates',AdminCMSListView),
            ('/paged/page/<int:page_num>','page_page',AdminListPageView),
            ('/paged/user/<int:page_num>','page_users',AdminListPageView),
            ('/paged/template/<int:page_num>','page_template',AdminListPageView),
            ('/paged/block/<int:page_num>','page_block',AdminListPageView),
            ('/view/block/<name>','block_view',AdminDetailView),
            ('/view/page/<slug>','page_view',AdminDetailView),
            ('/view/template/<name>','template_view',AdminDetailView),
            ('/edit/block/<item_id>','edit_block',AdminEditView),
            ('/edit/page/<item_id>','edit_page',AdminEditView),
            ('/edit/template/<item_id>','edit_template',AdminEditView),
            ('/edit/page/content/<item_id>','edit_page_content',AdminEditView),
            ('/edit/block/content/<item_id>','edit_block_content',AdminEditView),
            ('/pages','page_list',PageListView),
            ('/blogs','blogs',AdminBlogView),
        )
    ]



from ...baseviews import BaseView


class MenuView(BaseView):
    
    _template = ''
    _form = ''
    _context = {}

    def get(self):
        return self.render()


    def post(self):
        return self.render()



